var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = void 0;
const shared = require("@volar/shared");
const path = require("typesafe-path");
const vscode = require("vscode-languageserver");
const protocol_1 = require("../protocol");
const language_core_1 = require("@volar/language-core");
function register(connection, projects) {
    connection.onNotification(protocol_1.ReportStats.type, () => __awaiter(this, void 0, void 0, function* () {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
        for (const [rootUri, _workspace] of projects.workspaces) {
            connection.console.log('workspace: ' + rootUri);
            const workspace = yield _workspace;
            connection.console.log('documentRegistry stats: ' + workspace.documentRegistry.reportStats());
            connection.console.log('');
            connection.console.log('tsconfig: inferred');
            const _inferredProject = workspace.getInferredProjectDontCreate();
            if (_inferredProject) {
                connection.console.log('loaded: true');
                const inferredProject = yield _inferredProject;
                connection.console.log('largest 10 files:');
                for (const script of [...inferredProject.scripts.values()]
                    .sort((a, b) => { var _a, _b, _c, _d; return ((_b = (_a = b.snapshot) === null || _a === void 0 ? void 0 : _a.getLength()) !== null && _b !== void 0 ? _b : 0) - ((_d = (_c = a.snapshot) === null || _c === void 0 ? void 0 : _c.getLength()) !== null && _d !== void 0 ? _d : 0); })
                    .slice(0, 10)) {
                    connection.console.log('  - ' + script.fileName);
                    connection.console.log(`    size: ${(_a = script.snapshot) === null || _a === void 0 ? void 0 : _a.getLength()}`);
                }
                connection.console.log('files:');
                for (const script of inferredProject.scripts.values()) {
                    connection.console.log('  - ' + script.fileName);
                    connection.console.log(`    size: ${(_b = script.snapshot) === null || _b === void 0 ? void 0 : _b.getLength()}`);
                    connection.console.log(`    ref counts: "${(_d = (_c = workspace.documentRegistry).getLanguageServiceRefCounts) === null || _d === void 0 ? void 0 : _d.call(_c, script.fileName, (_f = (_e = inferredProject.languageServiceHost).getScriptKind) === null || _f === void 0 ? void 0 : _f.call(_e, script.fileName))})"`);
                }
            }
            else {
                connection.console.log('loaded: false');
            }
            connection.console.log('');
            for (const _project of workspace.projects.values()) {
                const project = yield _project;
                connection.console.log('tsconfig: ' + project.tsConfig);
                connection.console.log('loaded: ' + !!project.getLanguageServiceDontCreate());
                connection.console.log('largest 10 files:');
                for (const script of [...project.scripts.values()]
                    .sort((a, b) => { var _a, _b, _c, _d; return ((_b = (_a = b.snapshot) === null || _a === void 0 ? void 0 : _a.getLength()) !== null && _b !== void 0 ? _b : 0) - ((_d = (_c = a.snapshot) === null || _c === void 0 ? void 0 : _c.getLength()) !== null && _d !== void 0 ? _d : 0); })
                    .slice(0, 10)) {
                    connection.console.log('  - ' + script.fileName);
                    connection.console.log(`    size: ${(_g = script.snapshot) === null || _g === void 0 ? void 0 : _g.getLength()}`);
                }
                connection.console.log('files:');
                for (const script of project.scripts.values()) {
                    connection.console.log('  - ' + script.fileName);
                    connection.console.log(`    size: ${(_h = script.snapshot) === null || _h === void 0 ? void 0 : _h.getLength()}`);
                    connection.console.log(`    ref counts: "${(_k = (_j = workspace.documentRegistry).getLanguageServiceRefCounts) === null || _k === void 0 ? void 0 : _k.call(_j, script.fileName, (_m = (_l = project.languageServiceHost).getScriptKind) === null || _m === void 0 ? void 0 : _m.call(_l, script.fileName))})"`);
                }
            }
            connection.console.log('');
        }
    }));
    connection.onRequest(protocol_1.GetMatchTsConfigRequest.type, (params) => __awaiter(this, void 0, void 0, function* () {
        const project = (yield projects.getProject(params.uri));
        if (project === null || project === void 0 ? void 0 : project.tsconfig) {
            return { uri: shared.getUriByPath(project.tsconfig) };
        }
    }));
    connection.onRequest(protocol_1.GetVirtualFileNamesRequest.type, (document) => __awaiter(this, void 0, void 0, function* () {
        var _o, _p;
        const project = yield projects.getProject(document.uri);
        const fileNames = [];
        if (project) {
            const sourceFile = (_p = (_o = project.project) === null || _o === void 0 ? void 0 : _o.getLanguageService().context.core.mapper.get(shared.getPathOfUri(document.uri))) === null || _p === void 0 ? void 0 : _p[0];
            if (sourceFile) {
                (0, language_core_1.forEachEmbeddeds)(sourceFile, e => {
                    if (e.text && e.kind === 1) {
                        fileNames.push(e.fileName);
                    }
                });
            }
        }
        return fileNames;
    }));
    connection.onRequest(protocol_1.GetVirtualFileRequest.type, (params) => __awaiter(this, void 0, void 0, function* () {
        var _q, _r;
        const project = yield projects.getProject(params.sourceFileUri);
        if (project) {
            const embeddedFile = (_r = (_q = project.project) === null || _q === void 0 ? void 0 : _q.getLanguageService().context.core.mapper.fromEmbeddedFileName(params.virtualFileName)) === null || _r === void 0 ? void 0 : _r.embedded;
            if (embeddedFile) {
                return {
                    content: embeddedFile.text,
                    mappings: embeddedFile.mappings,
                };
            }
        }
    }));
    connection.onNotification(protocol_1.ReloadProjectNotification.type, () => {
        projects.reloadProject();
    });
    connection.onNotification(protocol_1.WriteVirtualFilesNotification.type, (params) => __awaiter(this, void 0, void 0, function* () {
        var _s;
        const fs = yield Promise.resolve().then(() => require('fs'));
        const project = yield projects.getProject(params.uri);
        if (project) {
            const ls = (_s = (yield project.project)) === null || _s === void 0 ? void 0 : _s.getLanguageServiceDontCreate();
            if (ls) {
                const sourceFiles = new Set(ls.context.host.getScriptFileNames());
                for (const virtualFile of ls.context.core.typescriptLanguageServiceHost.getScriptFileNames()) {
                    if (virtualFile.startsWith(ls.context.host.getCurrentDirectory()) && !sourceFiles.has(virtualFile)) {
                        const snapshot = ls.context.core.typescriptLanguageServiceHost.getScriptSnapshot(virtualFile);
                        if (snapshot) {
                            fs.writeFile(virtualFile, snapshot.getText(0, snapshot.getLength()), () => { });
                        }
                    }
                }
            }
        }
    }));
    connection.onNotification(protocol_1.VerifyAllScriptsNotification.type, (params) => __awaiter(this, void 0, void 0, function* () {
        var _t;
        let errors = 0;
        let warnings = 0;
        const progress = yield connection.window.createWorkDoneProgress();
        progress.begin('Verify', 0, '', true);
        const project = yield projects.getProject(params.uri);
        if (project) {
            const ls = (_t = (yield project.project)) === null || _t === void 0 ? void 0 : _t.getLanguageServiceDontCreate();
            if (ls) {
                const allVueDocuments = ls.context.documents.getAll();
                let i = 0;
                for (const vueFile of allVueDocuments) {
                    progress.report(i++ / allVueDocuments.length * 100, path.relative(ls.context.host.getCurrentDirectory(), shared.getPathOfUri(vueFile.uri)));
                    if (progress.token.isCancellationRequested) {
                        continue;
                    }
                    let _result = yield ls.doValidation(vueFile.uri, progress.token);
                    connection.sendDiagnostics({ uri: vueFile.uri, diagnostics: _result });
                    errors += _result.filter(error => error.severity === vscode.DiagnosticSeverity.Error).length;
                    warnings += _result.filter(error => error.severity === vscode.DiagnosticSeverity.Warning).length;
                }
            }
        }
        progress.done();
        connection.window.showInformationMessage(`Verification complete. Found ${errors} errors and ${warnings} warnings.`);
    }));
}
exports.register = register;
//# sourceMappingURL=customFeatures.js.map