var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = void 0;
const protocol_1 = require("../protocol");
function register(connection, documents, documentServiceHost) {
    connection.onDocumentFormatting(params => {
        return worker(params.textDocument.uri, document => {
            return documentServiceHost.get(document.uri).format(document, params.options);
        });
    });
    connection.onDocumentRangeFormatting(params => {
        return worker(params.textDocument.uri, document => {
            return documentServiceHost.get(document.uri).format(document, params.options, params.range);
        });
    });
    connection.onDocumentOnTypeFormatting(params => {
        return worker(params.textDocument.uri, document => {
            return documentServiceHost.get(document.uri).format(document, params.options, undefined, params);
        });
    });
    connection.onSelectionRanges(params => {
        return worker(params.textDocument.uri, document => {
            return documentServiceHost.get(document.uri).getSelectionRanges(document, params.positions);
        });
    });
    connection.onFoldingRanges(params => {
        return worker(params.textDocument.uri, document => {
            return documentServiceHost.get(document.uri).getFoldingRanges(document);
        });
    });
    connection.languages.onLinkedEditingRange(params => {
        return worker(params.textDocument.uri, document => {
            return documentServiceHost.get(document.uri).findLinkedEditingRanges(document, params.position);
        });
    });
    connection.onDocumentSymbol(params => {
        return worker(params.textDocument.uri, document => {
            return documentServiceHost.get(document.uri).findDocumentSymbols(document);
        });
    });
    connection.onDocumentColor(params => {
        return worker(params.textDocument.uri, document => {
            return documentServiceHost.get(document.uri).findDocumentColors(document);
        });
    });
    connection.onColorPresentation(params => {
        return worker(params.textDocument.uri, document => {
            return documentServiceHost.get(document.uri).getColorPresentations(document, params.color, params.range);
        });
    });
    connection.onRequest(protocol_1.AutoInsertRequest.type, (params) => __awaiter(this, void 0, void 0, function* () {
        return worker(params.textDocument.uri, document => {
            return documentServiceHost.get(document.uri).doAutoInsert(document, params.position, params.options);
        });
    }));
    function worker(uri, cb) {
        var _a;
        const document = (_a = documents.data.uriGet(uri)) === null || _a === void 0 ? void 0 : _a.getDocument();
        if (document) {
            return cb(document);
        }
    }
}
exports.register = register;
//# sourceMappingURL=documentFeatures.js.map