var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCommonLanguageServer = void 0;
const vscode = require("vscode-languageserver");
const vscode_uri_1 = require("vscode-uri");
const types_1 = require("./types");
const configurationHost_1 = require("./utils/configurationHost");
const documentServiceHost_1 = require("./utils/documentServiceHost");
const snapshots_1 = require("./utils/snapshots");
const workspaces_1 = require("./utils/workspaces");
const registerFeatures_1 = require("./registerFeatures");
const cancellationPipe_1 = require("./utils/cancellationPipe");
function createCommonLanguageServer(connection, runtimeEnv, _plugins) {
    let params;
    let options;
    let roots = [];
    let fsHost;
    let projects;
    let documentServiceHost;
    let configHost;
    let plugins;
    const documents = (0, snapshots_1.createSnapshots)(connection);
    connection.onInitialize((_params) => __awaiter(this, void 0, void 0, function* () {
        var _a, _b, _c, _d;
        params = _params;
        options = params.initializationOptions;
        plugins = _plugins.map(plugin => plugin(options));
        if (((_a = params.capabilities.workspace) === null || _a === void 0 ? void 0 : _a.workspaceFolders) && params.workspaceFolders) {
            roots = params.workspaceFolders.map(folder => vscode_uri_1.URI.parse(folder.uri));
        }
        else if (params.rootUri) {
            roots = [vscode_uri_1.URI.parse(params.rootUri)];
        }
        else if (params.rootPath) {
            roots = [vscode_uri_1.URI.file(params.rootPath)];
        }
        const result = {
            capabilities: {
                textDocumentSync: (_b = options.textDocumentSync) !== null && _b !== void 0 ? _b : vscode.TextDocumentSyncKind.Incremental,
            },
        };
        configHost = ((_c = params.capabilities.workspace) === null || _c === void 0 ? void 0 : _c.configuration) ? (0, configurationHost_1.createConfigurationHost)(params, connection) : undefined;
        const serverMode = (_d = options.serverMode) !== null && _d !== void 0 ? _d : types_1.ServerMode.Semantic;
        (0, registerFeatures_1.setupSyntacticCapabilities)(params.capabilities, result.capabilities, options);
        yield _createDocumentServiceHost();
        if (serverMode === types_1.ServerMode.Semantic) {
            (0, registerFeatures_1.setupSemanticCapabilities)(params.capabilities, result.capabilities, options, plugins);
            yield createLanguageServiceHost();
        }
        try {
            // show version on LSP logs
            const packageJson = require('../package.json');
            result.serverInfo = {
                name: packageJson.name,
                version: packageJson.version,
            };
        }
        catch (_e) { }
        return result;
    }));
    connection.onInitialized(() => __awaiter(this, void 0, void 0, function* () {
        var _f, _g, _h;
        fsHost === null || fsHost === void 0 ? void 0 : fsHost.ready(connection);
        configHost === null || configHost === void 0 ? void 0 : configHost.ready();
        if ((_f = params.capabilities.workspace) === null || _f === void 0 ? void 0 : _f.workspaceFolders) {
            connection.workspace.onDidChangeWorkspaceFolders(e => {
                for (const folder of e.added) {
                    documentServiceHost === null || documentServiceHost === void 0 ? void 0 : documentServiceHost.add(vscode_uri_1.URI.parse(folder.uri));
                    projects === null || projects === void 0 ? void 0 : projects.add(vscode_uri_1.URI.parse(folder.uri));
                }
                for (const folder of e.removed) {
                    documentServiceHost === null || documentServiceHost === void 0 ? void 0 : documentServiceHost.remove(vscode_uri_1.URI.parse(folder.uri));
                    projects === null || projects === void 0 ? void 0 : projects.remove(vscode_uri_1.URI.parse(folder.uri));
                }
            });
        }
        if (options.serverMode !== types_1.ServerMode.Syntactic
            && !options.disableFileWatcher
            && ((_h = (_g = params.capabilities.workspace) === null || _g === void 0 ? void 0 : _g.didChangeWatchedFiles) === null || _h === void 0 ? void 0 : _h.dynamicRegistration)) {
            connection.client.register(vscode.DidChangeWatchedFilesNotification.type, {
                watchers: [
                    ...plugins.map(plugin => plugin.extraFileExtensions.map(ext => ({ globPattern: `**/*.${ext.extension}` }))).flat(),
                    {
                        globPattern: `**/*.{${[
                            'js',
                            'cjs',
                            'mjs',
                            'ts',
                            'cts',
                            'mts',
                            'jsx',
                            'tsx',
                            'json',
                            ...plugins.map(plugin => plugin.extraFileExtensions.map(ext => ext.extension)).flat(),
                        ].join(',')}}`
                    },
                ]
            });
        }
    }));
    connection.listen();
    function _createDocumentServiceHost() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const ts = runtimeEnv.loadTypescript(options.typescript.tsdk);
            documentServiceHost = (0, documentServiceHost_1.createDocumentServiceHost)(runtimeEnv, plugins, ts, configHost, options);
            for (const root of roots) {
                documentServiceHost.add(root);
            }
            (yield Promise.resolve().then(() => require('./features/documentFeatures'))).register(connection, documents, documentServiceHost);
            for (const plugin of plugins) {
                (_b = (_a = plugin.syntacticService) === null || _a === void 0 ? void 0 : _a.onInitialize) === null || _b === void 0 ? void 0 : _b.call(_a, connection);
            }
        });
    }
    function createLanguageServiceHost() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const ts = runtimeEnv.loadTypescript(options.typescript.tsdk);
            fsHost = runtimeEnv.createFileSystemHost(ts, params.capabilities);
            const tsLocalized = params.locale ? yield runtimeEnv.loadTypescriptLocalized(options.typescript.tsdk, params.locale) : undefined;
            const cancelTokenHost = (0, cancellationPipe_1.createCancellationTokenHost)(options.cancellationPipeName);
            const _projects = (0, workspaces_1.createWorkspaces)(runtimeEnv, plugins, fsHost, configHost, ts, tsLocalized, params.capabilities, options, documents, connection, cancelTokenHost);
            projects = _projects;
            for (const root of roots) {
                projects.add(root);
            }
            (yield Promise.resolve().then(() => require('./features/customFeatures'))).register(connection, projects);
            (yield Promise.resolve().then(() => require('./features/languageFeatures'))).register(connection, projects, params, cancelTokenHost);
            for (const plugin of plugins) {
                (_b = (_a = plugin.semanticService) === null || _a === void 0 ? void 0 : _a.onInitialize) === null || _b === void 0 ? void 0 : _b.call(_a, connection, getLanguageService);
            }
            function getLanguageService(uri) {
                var _a;
                return __awaiter(this, void 0, void 0, function* () {
                    const project = (_a = (yield projects.getProject(uri))) === null || _a === void 0 ? void 0 : _a.project;
                    return project === null || project === void 0 ? void 0 : project.getLanguageService();
                });
            }
        });
    }
}
exports.createCommonLanguageServer = createCommonLanguageServer;
//# sourceMappingURL=server.js.map