/// <reference types="node" />
import * as embeddedLS from '@volar/language-service';
import * as embedded from '@volar/language-core';
import type { FileSystemProvider } from 'vscode-html-languageservice';
import type * as ts from 'typescript/lib/tsserverlibrary';
import * as vscode from 'vscode-languageserver';
import type * as Requests from './protocol';
import { URI } from 'vscode-uri';
export declare let __requests: typeof Requests;
export type FileSystemHost = {
    ready(connection: vscode.Connection): void;
    clearCache(): void;
    getWorkspaceFileSystem(rootUri: URI): FileSystem;
    onDidChangeWatchedFiles(cb: (params: vscode.DidChangeWatchedFilesParams) => void): () => void;
};
export type FileSystem = Pick<ts.System, 'newLine' | 'useCaseSensitiveFileNames' | 'fileExists' | 'readFile' | 'readDirectory' | 'getCurrentDirectory' | 'realpath' | 'resolvePath'> & Partial<ts.System>;
export interface RuntimeEnvironment {
    loadTypescript: (tsdk: string) => typeof import('typescript/lib/tsserverlibrary');
    loadTypescriptLocalized: (tsdk: string, locale: string) => Promise<{} | undefined>;
    schemaRequestHandlers: {
        [schema: string]: (uri: string, encoding?: BufferEncoding) => Promise<string>;
    };
    onDidChangeConfiguration?: (settings: any) => void;
    fileSystemProvide: FileSystemProvider | undefined;
    createFileSystemHost: (ts: typeof import('typescript/lib/tsserverlibrary'), capabilities: vscode.ClientCapabilities) => FileSystemHost;
}
export type LanguageServerPlugin<A extends LanguageServerInitializationOptions = LanguageServerInitializationOptions, B extends embedded.LanguageServiceHost = embedded.LanguageServiceHost, C = embeddedLS.LanguageService> = (initOptions: A) => {
    extraFileExtensions: ts.FileExtensionInfo[];
    semanticService?: {
        resolveLanguageServiceHost?(ts: typeof import('typescript/lib/tsserverlibrary'), sys: FileSystem, tsConfig: string | ts.CompilerOptions, host: embedded.LanguageServiceHost): B;
        getLanguageModules?(host: B): embedded.LanguageModule[];
        getServicePlugins?(host: B, service: embeddedLS.LanguageService): embeddedLS.LanguageServicePlugin[];
        onInitialize?(connection: vscode.Connection, getLanguageService: (uri: string) => Promise<C>): void;
    };
    syntacticService?: {
        getLanguageModules?(ts: typeof import('typescript/lib/tsserverlibrary'), env: embeddedLS.LanguageServicePluginContext['env']): embedded.LanguageModule[];
        getServicePlugins?(context: embeddedLS.DocumentServiceRuntimeContext): embeddedLS.LanguageServicePlugin[];
        onInitialize?(connection: vscode.Connection): void;
    };
};
export declare enum ServerMode {
    Semantic = 0,
    Syntactic = 2
}
export declare enum DiagnosticModel {
    None = 0,
    Push = 1,
    Pull = 2
}
export interface LanguageServerInitializationOptions {
    typescript: {
        tsdk: string;
    };
    serverMode?: ServerMode;
    diagnosticModel?: DiagnosticModel;
    textDocumentSync?: vscode.TextDocumentSyncKind | number;
    ignoreTriggerCharacters?: string[];
    /**
     * https://github.com/Microsoft/TypeScript/wiki/Standalone-Server-%28tsserver%29#cancellation
     */
    cancellationPipeName?: string;
    noProjectReferences?: boolean;
    reverseConfigFilePriority?: boolean;
    disableFileWatcher?: boolean;
    /**
     * Enable this option to make language server setup server capabilities based on client capabilities to support multiple servers.
     */
    respectClientCapabilities?: boolean;
    maxFileSize?: number;
    configFilePath?: string;
}
