Object.defineProperty(exports, "__esModule", { value: true });
exports.createDocumentServiceHost = void 0;
const embedded = require("@volar/language-service");
const vscode_uri_1 = require("vscode-uri");
const config_1 = require("./config");
function createDocumentServiceHost(runtimeEnv, plugins, ts, configHost, initOptions) {
    const workspaceServices = new Map();
    const untitledService = create(vscode_uri_1.URI.from({ scheme: 'untitled' }));
    return {
        add,
        remove,
        get,
    };
    function add(rootUri) {
        workspaceServices.set(rootUri.toString(), create(rootUri));
    }
    function remove(rootUri) {
        workspaceServices.delete(rootUri.toString());
    }
    function get(documentUri) {
        for (const [rootUri, service] of workspaceServices) {
            if (documentUri.startsWith(rootUri)) {
                return service;
            }
        }
        return untitledService;
    }
    function create(rootUri) {
        const env = {
            rootUri,
            configurationHost: configHost,
            fileSystemProvider: runtimeEnv.fileSystemProvide,
        };
        const serviceContext = embedded.createDocumentServiceContext({
            ts,
            env,
            getLanguageModules() {
                return plugins.map(plugin => { var _a, _b, _c; return (_c = (_b = (_a = plugin.syntacticService) === null || _a === void 0 ? void 0 : _a.getLanguageModules) === null || _b === void 0 ? void 0 : _b.call(_a, ts, env)) !== null && _c !== void 0 ? _c : []; }).flat();
            },
            getPlugins() {
                return [
                    ...(0, config_1.loadCustomPlugins)(rootUri.fsPath, initOptions.configFilePath),
                    ...plugins.map(plugin => { var _a, _b, _c; return (_c = (_b = (_a = plugin.syntacticService) === null || _a === void 0 ? void 0 : _a.getServicePlugins) === null || _b === void 0 ? void 0 : _b.call(_a, serviceContext)) !== null && _c !== void 0 ? _c : []; }).flat(),
                ];
            },
        });
        return embedded.createDocumentService(serviceContext);
    }
}
exports.createDocumentServiceHost = createDocumentServiceHost;
//# sourceMappingURL=documentServiceHost.js.map