var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createProject = void 0;
const shared = require("@volar/shared");
const embeddedLS = require("@volar/language-service");
const embedded = require("@volar/language-core");
const vscode = require("vscode-languageserver");
const config_1 = require("./config");
const path = require("typesafe-path");
const uriMap_1 = require("./uriMap");
function createProject(runtimeEnv, plugins, fsHost, ts, rootUri, tsConfig, tsLocalized, documents, configHost, documentRegistry, cancelTokenHost, serverOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        const sys = fsHost.getWorkspaceFileSystem(rootUri);
        let typeRootVersion = 0;
        let projectVersion = 0;
        let projectVersionUpdateTime = cancelTokenHost.getMtime();
        let vueLs;
        let parsedCommandLine = createParsedCommandLine(ts, sys, shared.getPathOfUri(rootUri.toString()), tsConfig, plugins);
        const scripts = (0, uriMap_1.createUriMap)();
        const languageServiceHost = createLanguageServiceHost();
        const disposeWatchEvent = fsHost.onDidChangeWatchedFiles(params => {
            onWorkspaceFilesChanged(params.changes);
        });
        const disposeDocChange = documents.onDidChangeContent(() => {
            projectVersion++;
            projectVersionUpdateTime = cancelTokenHost.getMtime();
        });
        return {
            tsConfig,
            scripts,
            languageServiceHost,
            getLanguageService,
            getLanguageServiceDontCreate: () => vueLs,
            getParsedCommandLine: () => parsedCommandLine,
            tryAddFile: (fileName) => {
                if (!parsedCommandLine.fileNames.includes(fileName)) {
                    parsedCommandLine.fileNames.push(fileName);
                    projectVersion++;
                }
            },
            dispose,
        };
        function getLanguageService() {
            if (!vueLs) {
                const languageModules = plugins.map(plugin => { var _a, _b, _c; return (_c = (_b = (_a = plugin.semanticService) === null || _a === void 0 ? void 0 : _a.getLanguageModules) === null || _b === void 0 ? void 0 : _b.call(_a, languageServiceHost)) !== null && _c !== void 0 ? _c : []; }).flat();
                const languageContext = embedded.createEmbeddedLanguageServiceHost(languageServiceHost, languageModules);
                const languageServiceContext = embeddedLS.createLanguageServiceContext({
                    host: languageServiceHost,
                    context: languageContext,
                    getPlugins() {
                        return [
                            ...(0, config_1.loadCustomPlugins)(languageServiceHost.getCurrentDirectory(), serverOptions.configFilePath),
                            ...plugins.map(plugin => { var _a, _b, _c; return (_c = (_b = (_a = plugin.semanticService) === null || _a === void 0 ? void 0 : _a.getServicePlugins) === null || _b === void 0 ? void 0 : _b.call(_a, languageServiceHost, vueLs)) !== null && _c !== void 0 ? _c : []; }).flat(),
                        ];
                    },
                    env: {
                        rootUri,
                        configurationHost: configHost,
                        fileSystemProvider: runtimeEnv.fileSystemProvide,
                        documentContext: getHTMLDocumentContext(ts, languageServiceHost),
                        schemaRequestService: (uri) => __awaiter(this, void 0, void 0, function* () {
                            const protocol = uri.substring(0, uri.indexOf(':'));
                            const builtInHandler = runtimeEnv.schemaRequestHandlers[protocol];
                            if (builtInHandler) {
                                return yield builtInHandler(uri);
                            }
                            return '';
                        }),
                    },
                    documentRegistry,
                });
                vueLs = embeddedLS.createLanguageService(languageServiceContext);
            }
            return vueLs;
        }
        function onWorkspaceFilesChanged(changes) {
            return __awaiter(this, void 0, void 0, function* () {
                for (const change of changes) {
                    const script = scripts.uriGet(change.uri);
                    if (script && (change.type === vscode.FileChangeType.Changed || change.type === vscode.FileChangeType.Created)) {
                        if (script.version >= 0) {
                            script.version = -1;
                        }
                        else {
                            script.version--;
                        }
                    }
                    else if (script && change.type === vscode.FileChangeType.Deleted) {
                        scripts.uriDelete(change.uri);
                    }
                    if (script) {
                        projectVersion++;
                    }
                }
                const creates = changes.filter(change => change.type === vscode.FileChangeType.Created);
                const deletes = changes.filter(change => change.type === vscode.FileChangeType.Deleted);
                if (creates.length || deletes.length) {
                    parsedCommandLine = createParsedCommandLine(ts, sys, shared.getPathOfUri(rootUri.toString()), tsConfig, plugins);
                    projectVersion++;
                    typeRootVersion++;
                }
            });
        }
        function createLanguageServiceHost() {
            var _a;
            const token = {
                isCancellationRequested() {
                    return cancelTokenHost.getMtime() !== projectVersionUpdateTime;
                },
                throwIfCancellationRequested() { },
            };
            let host = {
                // ts
                getNewLine: () => sys.newLine,
                useCaseSensitiveFileNames: () => sys.useCaseSensitiveFileNames,
                readFile: sys.readFile,
                writeFile: sys.writeFile,
                directoryExists: sys.directoryExists,
                getDirectories: sys.getDirectories,
                readDirectory: sys.readDirectory,
                realpath: sys.realpath,
                fileExists: sys.fileExists,
                getCurrentDirectory: () => shared.getPathOfUri(rootUri.toString()),
                getProjectReferences: () => parsedCommandLine.projectReferences,
                getCancellationToken: () => token,
                // custom
                getDefaultLibFileName: options => {
                    try {
                        return ts.getDefaultLibFilePath(options);
                    }
                    catch (_a) {
                        // web
                        return serverOptions.typescript.tsdk + '/' + ts.getDefaultLibFileName(options);
                    }
                },
                getProjectVersion: () => projectVersion.toString(),
                getTypeRootsVersion: () => typeRootVersion,
                getScriptFileNames: () => parsedCommandLine.fileNames,
                getCompilationSettings: () => parsedCommandLine.options,
                getScriptVersion,
                getScriptSnapshot,
                getTypeScriptModule: () => ts,
            };
            if (serverOptions.noProjectReferences) {
                host.getProjectReferences = undefined;
                host.getCompilationSettings = () => (Object.assign(Object.assign({}, parsedCommandLine.options), { rootDir: undefined, composite: false }));
            }
            if (tsLocalized) {
                host.getLocalizedDiagnosticMessages = () => tsLocalized;
            }
            for (const plugin of plugins) {
                if ((_a = plugin.semanticService) === null || _a === void 0 ? void 0 : _a.resolveLanguageServiceHost) {
                    host = plugin.semanticService.resolveLanguageServiceHost(ts, sys, tsConfig, host);
                }
            }
            return host;
            function getScriptVersion(fileName) {
                var _a, _b;
                const doc = documents.data.pathGet(fileName);
                if (doc) {
                    return doc.version.toString();
                }
                return (_b = (_a = scripts.pathGet(fileName)) === null || _a === void 0 ? void 0 : _a.version.toString()) !== null && _b !== void 0 ? _b : '';
            }
            function getScriptSnapshot(fileName) {
                var _a;
                const doc = documents.data.pathGet(fileName);
                if (doc) {
                    return doc.getSnapshot();
                }
                const script = scripts.pathGet(fileName);
                if (script && script.snapshotVersion === script.version) {
                    return script.snapshot;
                }
                if (sys.fileExists(fileName)) {
                    if (serverOptions.maxFileSize) {
                        const fileSize = (_a = sys.getFileSize) === null || _a === void 0 ? void 0 : _a.call(sys, fileName);
                        if (fileSize !== undefined && fileSize > serverOptions.maxFileSize) {
                            console.warn(`IGNORING "${fileName}" because it is too large (${fileSize}bytes > ${serverOptions.maxFileSize}bytes)`);
                            return ts.ScriptSnapshot.fromString('');
                        }
                    }
                    const text = sys.readFile(fileName, 'utf8');
                    if (text !== undefined) {
                        const snapshot = ts.ScriptSnapshot.fromString(text);
                        if (script) {
                            script.snapshot = snapshot;
                            script.snapshotVersion = script.version;
                        }
                        else {
                            scripts.pathSet(fileName, {
                                version: -1,
                                fileName: fileName,
                                snapshot: snapshot,
                                snapshotVersion: -1,
                            });
                        }
                        return snapshot;
                    }
                }
            }
        }
        function dispose() {
            vueLs === null || vueLs === void 0 ? void 0 : vueLs.dispose();
            scripts.clear();
            disposeWatchEvent();
            disposeDocChange();
        }
    });
}
exports.createProject = createProject;
function createParsedCommandLine(ts, sys, rootPath, tsConfig, plugins) {
    const extraFileExtensions = plugins.map(plugin => plugin.extraFileExtensions).flat();
    try {
        let content;
        if (typeof tsConfig === 'string') {
            const config = ts.readJsonConfigFile(tsConfig, sys.readFile);
            content = ts.parseJsonSourceFileConfigFileContent(config, sys, path.dirname(tsConfig), {}, tsConfig, undefined, extraFileExtensions);
        }
        else {
            content = ts.parseJsonConfigFileContent({ files: [] }, sys, rootPath, tsConfig, path.join(rootPath, 'jsconfig.json'), undefined, extraFileExtensions);
        }
        // fix https://github.com/johnsoncodehk/volar/issues/1786
        // https://github.com/microsoft/TypeScript/issues/30457
        // patching ts server broke with outDir + rootDir + composite/incremental
        content.options.outDir = undefined;
        content.fileNames = content.fileNames.map(shared.normalizeFileName);
        return content;
    }
    catch (_a) {
        // will be failed if web fs host first result not ready
        return {
            errors: [],
            fileNames: [],
            options: {},
        };
    }
}
function getHTMLDocumentContext(ts, host) {
    const documentContext = {
        resolveReference(ref, base) {
            var _a, _b;
            const isUri = base.indexOf('://') >= 0;
            const resolveResult = ts.resolveModuleName(ref, isUri ? shared.getPathOfUri(base) : base, host.getCompilationSettings(), host);
            const failedLookupLocations = resolveResult.failedLookupLocations;
            const dirs = new Set();
            for (let failed of failedLookupLocations) {
                const fileName = path.basename(failed);
                if (fileName === 'index.d.ts' || fileName === '*.d.ts') {
                    dirs.add(path.dirname(failed));
                }
                if (failed.endsWith('.d.ts')) {
                    failed = failed.substring(0, failed.length - '.d.ts'.length);
                }
                else {
                    continue;
                }
                if (host.fileExists(failed)) {
                    return isUri ? shared.getUriByPath(failed) : failed;
                }
            }
            for (const dir of dirs) {
                if ((_b = (_a = host.directoryExists) === null || _a === void 0 ? void 0 : _a.call(host, dir)) !== null && _b !== void 0 ? _b : true) {
                    return isUri ? shared.getUriByPath(dir) : dir;
                }
            }
            return undefined;
        },
    };
    return documentContext;
}
//# sourceMappingURL=project.js.map